#!/bin/bash
############################################################################
#                    UNISIM TMS320C3X testing environment                  #
#                             -------------------                          #
#    begin            : Fri July 1st 2009                                  #
#    copyright        : (C) 2009 Commissariat a l'Energie Atomique (CEA)   #
#    authors          : Gilles Mouchard, Daniel Gracia Pérez               #
#    email            : gilles.mouchard@cea.fr, daniel.gracia-perez@cea.fr #
############################################################################

############################################################################
#                                                                          #
#    This program is free software; you can redistribute it and/or modify  #
#    it under the terms of the GNU General Public License version 2        #
#    as published by the Free Software Foundation.                         #
#                                                                          #
############################################################################

# Note: The test plan is located in function 'factorial'
#       The goal of that function is to generate an auxiliary Makefile (Makefile.aux)
#       that contains building rules of planned unit tests

unit_tests_count=0
instruction_count=0
next_unit_tests_count=100

function factorial
{
	printf ""
	memory_instructions="yes" # "yes" or "no"
	control_instructions="yes"
	integer_2ops_instructions="yes"
	integer_3ops_instructions="yes"
	integer_parallel_instructions="yes"
	float_2ops_instructions="yes"
	float_3ops_instructions="yes"
	float_parallel_instruction="yes"

##########################################################
# start: memory instructions                             #

	local -a conds=("u" "lo" "ls" "hi" "hs" "eq" "ne" "lt" "le" "gt" "ge" "nv" "v" "nuf" "uf" "nlv" "lv" "nluf" "luf" "zuf")

	if [ "${memory_instructions}" = "yes" ]; then

		for insn in ldi; do
			generate 100 2ops_src_int_reg_dst_int_reg.pat loadstore_int_reg/${insn} ${insn}
			generate_src_int_indir_dst_int_reg 100 loadstore_int_indir ${insn}
			generate_src_int_imm_dst_int_reg 1 loadstore_int_imm ${insn}
			generate 100 src_int_dir_dst_int_reg.pat loadstore_int_dir/${insn} ${insn}
			generate 100 2ops_src_int_buf_dst_int_reg_ar_update_ordering.pat loadstore_int_indir/${insn}_ar_update_ordering ${insn}
		done

		for cond in ${conds[@]}; do
			for insn in ldi; do
				generate 100 2ops_src_int_reg_src_dst_int_reg.pat loadstore_int_reg/${insn}${cond} ${insn}${cond}
				generate_src_int_indir_src_dst_int_reg 100 loadstore_int_indir ${insn}${cond}
				generate_src_int_imm_src_dst_int_reg 1 loadstore_int_imm ${insn}${cond}
				generate 100 src_int_dir_src_dst_int_reg.pat loadstore_int_dir/${insn}${cond} ${insn}${cond}
				generate 100 2ops_src_int_buf_dst_int_reg_ar_update_ordering.pat loadstore_int_indir/${insn}${cond}_ar_update_ordering ${insn}${cond}
			done
		done

		for insn in sti; do
			generate_src_int_reg_dst_int_indir 100 loadstore_int_indir ${insn}
			generate 100 src_int_reg_dst_int_dir.pat loadstore_int_dir/${insn} ${insn}
			generate 100 2ops_src_int_reg_dst_int_buf_ar_update_ordering.pat loadstore_int_indir/${insn}_ar_update_ordering ${insn}
		done

		for insn in lde ldm; do
			generate 100 2ops_src_float_reg_src_dst_float_reg.pat loadstore_float_reg/${insn} ${insn}
			generate_src_float_indir_src_dst_float_reg 100 loadstore_float_indir ${insn}
			generate_src_float_imm_dst_float_reg 1 2ops_src_float_imm_src_dst_float_reg.pat loadstore_float_imm/${insn} ${insn}
			generate 100 src_float_dir_src_dst_float_reg.pat loadstore_float_dir/${insn} ${insn}
		done

		for insn in ldf; do
			generate 100 2ops_src_float_reg_dst_float_reg.pat loadstore_float_reg/${insn} ${insn}
			generate_src_float_indir_dst_float_reg 100 loadstore_float_indir ${insn}
			generate_src_float_imm_dst_float_reg 1 2ops_src_float_imm_dst_float_reg.pat loadstore_float_imm/${insn} ${insn}
			generate 100 src_float_dir_dst_float_reg.pat loadstore_float_dir/${insn} ${insn}
		done

		for cond in ${conds[@]}; do
			for insn in ldf; do
				generate 100 2ops_src_float_reg_src_dst_float_reg.pat loadstore_float_reg/${insn}${cond} ${insn}${cond}
				generate_src_float_indir_src_dst_float_reg 100 loadstore_float_indir ${insn}${cond}
				generate_src_float_imm_dst_float_reg 1 2ops_src_float_imm_src_dst_float_reg.pat loadstore_float_imm/${insn}${cond} ${insn}${cond}
				generate 100 src_float_dir_src_dst_float_reg.pat loadstore_float_dir/${insn}${cond} ${insn}${cond}
			done
		done

		for insn in stf; do
			generate_src_float_reg_dst_float_indir 100 loadstore_float_indir ${insn}
			generate 100 src_float_reg_dst_float_dir.pat loadstore_float_dir/${insn} ${insn}
		done

	fi

# end: memory instructions                               #
##########################################################

##########################################################
# start: control instructions                            #

	if [ "${control_instructions}" = "yes" ]; then

		for cond in ${conds[@]}; do
			for insn in b; do
				generate 100 cond_branch_disp.pat control_disp/${insn}${cond} ${insn}${cond}
				generate 100 cond_branch_reg.pat control_reg/${insn}${cond} ${insn}${cond}
			done

			for insn in b; do
				generate 100 cond_delayed_branch_disp.pat control_disp/${insn}${cond}d ${insn}${cond}d
				generate 100 cond_delayed_branch_reg.pat control_reg/${insn}${cond}d ${insn}${cond}d
			done

			for insn in db; do
				generate 100 dec_cond_branch_disp.pat control_disp/${insn}${cond} ${insn}${cond}
				generate 100 dec_cond_branch_reg.pat control_reg/${insn}${cond} ${insn}${cond}
			done

			for insn in db; do
				generate 100 dec_cond_delayed_branch_disp.pat control_disp/${insn}${cond}d ${insn}${cond}d
				generate 100 dec_cond_delayed_branch_reg.pat control_reg/${insn}${cond}d ${insn}${cond}d
			done

			for insn in call; do
				generate 100 cond_call_disp.pat control_disp/${insn}${cond} ${insn}${cond}
				generate 100 cond_call_reg.pat control_reg/${insn}${cond} ${insn}${cond}
			done

			for insn in rets; do
				generate 100 cond_return.pat control_abs/${insn}${cond} ${insn}${cond}
			done
		done

		for insn in br; do
			generate 100 uncond_branch.pat control_abs/${insn} ${insn}
		done

		for insn in brd; do
			generate 100 uncond_delayed_branch.pat control_abs/${insn} ${insn}
		done

		for insn in call; do
			generate 100 call.pat control_abs/${insn} ${insn}
		done

		for cond in ${conds[@]}; do
			for n in 0; do
				for insn in trap; do
					generate 100 cond_trap.pat control_imm/${insn}${cond}/${n} ${n} ${insn}${cond}
				done
			done
		done

		for cond in u; do
			for n in 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27; do
				for insn in trap; do
					generate 100 cond_trap.pat control_imm/${insn}${cond}/${n} ${n} ${insn}${cond}
				done
			done
		done

		for cond in ${conds[@]}; do
			for insn in reti; do
				generate 100 cond_reti.pat control_abs/${insn}${cond} ${insn}${cond}
			done
		done

		for insn in rpts; do
			generate 100 rpts_reg.pat control_reg/${insn}
 			generate 100 rpts_dir.pat control_dir/${insn}
 			generate 100 rpts_indir.pat control_indir/${insn}
 			generate 100 rpts_imm.pat control_imm/${insn} 10
		done

		for insn in rptb; do
			generate 100 rptb.pat control_abs/${insn}
		done

	fi

# end: control instructions                              #
##########################################################
	
##########################################################
# start: integer 2 operands instructions                 #

	if [ "${integer_2ops_instructions}" = "yes" ]; then

		for insn in absi negb negi not; do
			generate 10000 2ops_src_int_reg_dst_int_reg.pat 2ops_int_reg/${insn} ${insn}
			generate_src_int_indir_dst_int_reg 100 2ops_int_indir ${insn}
			generate 100 src_int_dir_dst_int_reg.pat 2ops_int_dir/${insn} ${insn}
			generate_src_int_imm_dst_int_reg 1 2ops_int_imm ${insn}
		done

		for insn in cmpi tstb; do
			generate 10000 2ops_src_int_reg_src_int_reg.pat 2ops_int_reg/${insn} ${insn}
			generate_src_int_indir_src_int_reg 100 2ops_int_indir ${insn}
			generate 100 src_int_dir_src_int_reg.pat 2ops_int_dir/${insn} ${insn}
			generate_src_int_imm_src_int_reg 100 2ops_int_imm ${insn}
		done

		for insn in addc addi and andn ash lsh mpyi or subb subc subi subrb subri xor; do
			generate 10000 2ops_src_int_reg_src_dst_int_reg.pat 2ops_int_reg/${insn} ${insn}
			generate_src_int_indir_src_dst_int_reg 100 2ops_int_indir ${insn}
			generate 100 src_int_dir_src_dst_int_reg.pat 2ops_int_dir/${insn} ${insn}
			generate_src_int_imm_src_dst_int_reg 100 2ops_int_imm ${insn}
		done

		for insn in rol rolc ror rorc; do
			generate 10000 rot_src_dst_int_reg.pat 2ops_int_reg/${insn} ${insn}
		done

		for insn in absi negb negi not cmpi tstb addc addi and andn ash lsh mpyi or subb subc subi subrb subri xor; do
			generate 100 2ops_src_int_buf_dst_int_reg_ar_update_ordering.pat 2ops_int_buf/${insn}_ar_update_ordering ${insn}
		done

	fi

# end: integer 2 operands instructions                   #
##########################################################

##########################################################
# START: integer 3 operands instructions                 #

	if [ "${integer_3ops_instructions}" = "yes" ]; then

		for insn in addi3 and3 andn3 ash3 lsh3 mpyi3 or3 subb3 subi3 xor3; do 
			# reg reg
			generate 100 3ops_src_int_reg_src_int_reg_dst_int_reg.pat 3ops_int_reg_reg/${insn} ${insn}
			# reg buf
			generate 100 3ops_src_int_reg_src_int_buf_dst_int_reg.pat 3ops_int_reg_buf/${insn} ${insn}
			generate 100 3ops_src_int_reg_src_int_buf_dst_int_reg_ar_update_ordering.pat 3ops_int_reg_buf/${insn}_ar_update_ordering ${insn}
			# buf reg
			generate 100 3ops_src_int_buf_src_int_reg_dst_int_reg.pat 3ops_int_buf_reg/${insn} ${insn}
			generate 100 3ops_src_int_buf_src_int_reg_dst_int_reg_ar_update_ordering.pat 3ops_int_buf_reg/${insn}_ar_update_ordering ${insn}
			# buf buf
			generate 100 3ops_src_int_buf_src_int_buf_dst_int_reg.pat 3ops_int_buf_buf/${insn} ${insn}
			generate 100 3ops_src_int_buf_src_int_buf_dst_int_reg_ar_update_ordering.pat 3ops_int_buf_buf/${insn}_ar_update_ordering ${insn}
		done

		for insn in cmpi3 tstb3; do 
			# reg reg
			generate 100 3ops_src_int_reg_src_int_reg.pat 3ops_int_reg_reg/${insn} ${insn}
			# reg buf
			generate 100 3ops_src_int_reg_src_int_buf.pat 3ops_int_reg_buf/${insn} ${insn}
			generate 100 3ops_src_int_reg_src_int_buf_ar_update_ordering.pat 3ops_int_reg_buf/${insn}_ar_update_ordering ${insn}
			# buf reg
			generate 100 3ops_src_int_buf_src_int_reg.pat 3ops_int_buf_reg/${insn} ${insn}
			generate 100 3ops_src_int_buf_src_int_reg_ar_update_ordering.pat 3ops_int_buf_reg/${insn}_ar_update_ordering ${insn}
			# buf buf
			generate 100 3ops_src_int_buf_src_int_buf.pat 3ops_int_buf_buf/${insn} ${insn}
			generate 100 3ops_src_int_buf_src_int_buf_ar_update_ordering.pat 3ops_int_buf_buf/${insn}_ar_update_ordering ${insn}
		done

	fi

# END: integer 3 operands instructions                   #
##########################################################

##########################################################
# start: integer parallel instructions                   #
	
	if [ "${integer_parallel_instructions}" = "yes" ]; then

		# ldi || ldi
		# buf_buf
		generate 100 par_src_int_buf_dst_int_reg_src_int_buf_dst_int_reg.pat parallel_int/ldi_ldi/buf_buf ldi ldi
		generate 100 par_src_int_buf_dst_int_reg_src_int_buf_dst_int_reg_ar_update_ordering.pat parallel_int/ldi_ldi_ar_update_ordering/buf_buf ldi ldi
		# reg_buf
		generate 100 par_src_int_reg_dst_int_reg_src_int_buf_dst_int_reg.pat parallel_int/ldi_ldi/reg_buf ldi ldi
		generate 100 par_src_int_reg_dst_int_reg_src_int_buf_dst_int_reg_ar_update_ordering.pat parallel_int/ldi_ldi_ar_update_ordering/reg_buf ldi ldi

		# ldi || sti
		# buf
		generate 100 par_src_int_buf_dst_int_reg_src_int_reg_dst_int_buf.pat parallel_int/ldi_sti/buf_buf ldi sti
		generate 100 par_src_int_buf_dst_int_reg_src_int_reg_dst_int_buf_ar_update_ordering.pat parallel_int/ldi_sti_ar_update_ordering/buf_buf ldi sti
		# reg
		generate 100 par_src_int_reg_dst_int_reg_src_int_reg_dst_int_buf.pat parallel_int/ldi_sti/reg_buf ldi sti
		generate 100 par_src_int_reg_dst_int_reg_src_int_reg_dst_int_buf_ar_update_ordering.pat parallel_int/ldi_sti_ar_update_ordering/reg_buf ldi sti

		# sti || sti
		# buf_buf
		generate 100 par_src_int_reg_dst_int_buf_src_int_reg_dst_int_buf.pat parallel_int/sti_sti/buf_buf sti sti
		generate 100 par_src_int_reg_dst_int_buf_src_int_reg_dst_int_buf_ar_update_ordering.pat parallel_int/sti_sti_ar_update_ordering/buf_buf sti sti
		# reg_buf
		generate 100 par_src_int_reg_dst_int_reg_src_int_reg_dst_int_buf.pat parallel_int/sti_sti/reg_buf sti sti
		generate 100 par_src_int_reg_dst_int_reg_src_int_reg_dst_int_buf_ar_update_ordering.pat parallel_int/sti_sti_ar_update_ordering/reg_buf sti sti

		for insn in absi negi not; do
			for insn2 in sti; do
				# buf reg
				generate 100 par_src_int_buf_dst_int_reg_src_int_reg_dst_int_buf.pat parallel_int/${insn}_${insn2}/buf_reg ${insn} ${insn2}
				generate 100 par_src_int_buf_dst_int_reg_src_int_reg_dst_int_buf_ar_update_ordering.pat parallel_int/${insn}_${insn2}_ar_update_ordering/buf_reg ${insn} ${insn2}

				# reg reg
				generate 100 par_src_int_reg_dst_int_reg_src_int_reg_dst_int_buf.pat parallel_int/${insn}_${insn2}/reg_reg ${insn} ${insn2}
				generate 100 par_src_int_reg_dst_int_reg_src_int_reg_dst_int_buf_ar_update_ordering.pat parallel_int/${insn}_${insn2}_ar_update_ordering/reg_reg ${insn} ${insn2}
			done
		done

		for insn in addi3 and3 ash3 lsh3 mpyi3 or3 subi3 xor3; do
			for insn2 in sti; do
				# reg buf
				generate 100 par_src_int_reg_src_int_buf_dst_int_reg_src_int_reg_dst_int_buf.pat parallel_int/${insn}_${insn2}/reg_buf ${insn} ${insn2}
				generate 100 par_src_int_reg_src_int_buf_dst_int_reg_src_int_reg_dst_int_buf_ar_update_ordering.pat parallel_int/${insn}_${insn2}_ar_update_ordering/reg_buf ${insn} ${insn2}

				# reg reg
				generate 100 par_src_int_reg_src_int_reg_dst_int_reg_src_int_reg_dst_int_buf.pat parallel_int/${insn}_${insn2}/reg_reg ${insn} ${insn2}
				generate 100 par_src_int_reg_src_int_reg_dst_int_reg_src_int_reg_dst_int_buf_ar_update_ordering.pat parallel_int/${insn}_${insn2}_ar_update_ordering/reg_reg ${insn} ${insn2}
			done
		done

		for insn in mpyi3; do
			for insn2 in addi3 subi3; do
				# buf_buf_reg_reg
				generate 100 par_src_int_buf_src_int_buf_dst_int_reg_src_int_reg_src_int_reg_dst_int_reg.pat parallel_int/${insn}_${insn2}/buf_buf_reg_reg ${insn} ${insn2}
				generate 100 par_src_int_buf_src_int_buf_dst_int_reg_src_int_reg_src_int_reg_dst_int_reg_ar_update_ordering.pat parallel_int/${insn}_${insn2}_ar_update_ordering/buf_buf_reg_reg ${insn} ${insn2}
				# buf_reg_reg_reg
				generate 100 par_src_int_buf_src_int_reg_dst_int_reg_src_int_reg_src_int_reg_dst_int_reg.pat parallel_int/${insn}_${insn2}/buf_reg_reg_reg ${insn} ${insn2}
				generate 100 par_src_int_buf_src_int_reg_dst_int_reg_src_int_reg_src_int_reg_dst_int_reg_ar_update_ordering.pat parallel_int/${insn}_${insn2}_ar_update_ordering/buf_reg_reg_reg ${insn} ${insn2}
				# reg_buf_reg_reg
				generate 100 par_src_int_reg_src_int_buf_dst_int_reg_src_int_reg_src_int_reg_dst_int_reg.pat parallel_int/${insn}_${insn2}/reg_buf_reg_reg ${insn} ${insn2}
				generate 100 par_src_int_reg_src_int_buf_dst_int_reg_src_int_reg_src_int_reg_dst_int_reg_ar_update_ordering.pat parallel_int/${insn}_${insn2}_ar_update_ordering/reg_buf_reg_reg ${insn} ${insn2}
				# reg_reg_reg_reg
				generate 100 par_src_int_reg_src_int_reg_dst_int_reg_src_int_reg_src_int_reg_dst_int_reg.pat parallel_int/${insn}_${insn2}/reg_reg_reg_reg ${insn} ${insn2}
				# buf_reg_buf_reg
				generate 100 par_src_int_buf_src_int_reg_dst_int_reg_src_int_buf_src_int_reg_dst_int_reg.pat parallel_int/${insn}_${insn2}/buf_reg_buf_reg ${insn} ${insn2}
				generate 100 par_src_int_buf_src_int_reg_dst_int_reg_src_int_buf_src_int_reg_dst_int_reg_ar_update_ordering.pat parallel_int/${insn}_${insn2}_ar_update_ordering/buf_reg_buf_reg ${insn} ${insn2}
				# reg_reg_buf_reg
				generate 100 par_src_int_reg_src_int_reg_dst_int_reg_src_int_buf_src_int_reg_dst_int_reg.pat parallel_int/${insn}_${insn2}/reg_reg_buf_reg ${insn} ${insn2}
				generate 100 par_src_int_reg_src_int_reg_dst_int_reg_src_int_buf_src_int_reg_dst_int_reg_ar_update_ordering.pat parallel_int/${insn}_${insn2}_ar_update_ordering/reg_reg_buf_reg ${insn} ${insn2}
				# reg_reg_buf_buf
				generate 100 par_src_int_reg_src_int_reg_dst_int_reg_src_int_buf_src_int_buf_dst_int_reg.pat parallel_int/${insn}_${insn2}/reg_reg_buf_buf ${insn} ${insn2} 
				generate 100 par_src_int_reg_src_int_reg_dst_int_reg_src_int_buf_src_int_buf_dst_int_reg_ar_update_ordering.pat parallel_int/${insn}_${insn2}_ar_update_ordering/reg_reg_buf_buf ${insn} ${insn2} 
				# reg_reg_reg_buf
				generate 100 par_src_int_reg_src_int_reg_dst_int_reg_src_int_reg_src_int_buf_dst_int_reg.pat parallel_int/${insn}_${insn2}/reg_reg_reg_buf ${insn} ${insn2} 
				generate 100 par_src_int_reg_src_int_reg_dst_int_reg_src_int_reg_src_int_buf_dst_int_reg_ar_update_ordering.pat parallel_int/${insn}_${insn2}_ar_update_ordering/reg_reg_reg_buf ${insn} ${insn2} 
				# buf_reg_reg_buf
				generate 100 par_src_int_buf_src_int_reg_dst_int_reg_src_int_reg_src_int_buf_dst_int_reg.pat parallel_int/${insn}_${insn2}/buf_reg_reg_buf ${insn} ${insn2} 
				generate 100 par_src_int_buf_src_int_reg_dst_int_reg_src_int_reg_src_int_buf_dst_int_reg_ar_update_ordering.pat parallel_int/${insn}_${insn2}_ar_update_ordering/buf_reg_reg_buf ${insn} ${insn2} 
			done
		done

	fi

# END: integer parallel instructions                     #
##########################################################

##########################################################
# start: float 2 operands instructions                   #
	
	int_as_float_tests="no"
	if [ "${int_as_float_tests}" = "yes" ]; then
	for insn in absf negf norm rnd; do
		generate 100 test_2ops_src_int_reg_dst_int_reg.pat test_2ops_float_reg/int_int/${insn} ${insn}
		generate 100 test_2ops_src_float_reg_dst_int_reg.pat test_2ops_float_reg/float_int/${insn} ${insn}
		generate 100 test_2ops_src_int_reg_dst_float_reg.pat test_2ops_float_reg/int_float/${insn} ${insn}
		generate 100 test_2ops_src_float_reg_dst_float_reg.pat test_2ops_float_reg/float_float/${insn} ${insn}
		generate_test_src_float_indir_src_dst_int_reg 100 test_2ops_float_indir ${insn}
		generate_test_src_float_imm_dst_float_reg 100 test_2ops_src_float_imm_src_dst_int_reg.pat test_2ops_float_imm/${insn} ${insn}
		generate 100 test_src_float_dir_src_dst_int_reg.pat test_2ops_float_dir/${insn} ${insn}
	done

	for insn in addf subf subrf mpyf; do
		generate 100 test_2ops_float_reg_src_int_reg_src_dst_int_reg.pat test_2ops_float_reg/int_int/${insn} ${insn}
		generate 100 test_2ops_float_reg_src_float_reg_src_dst_int_reg.pat test_2ops_float_reg/float_int/${insn} ${insn}
		generate 100 test_2ops_float_reg_src_int_reg_src_dst_float_reg.pat test_2ops_float_reg/int_float/${insn} ${insn}
		generate 100 test_2ops_float_reg_src_float_reg_src_dst_float_reg.pat test_2ops_float_reg/float_float/${insn} ${insn}
		generate_test_src_float_indir_src_dst_int_reg 100 test_2ops_float_indir ${insn}
		generate_test_src_float_imm_dst_float_reg 100 test_2ops_src_float_imm_src_dst_int_reg.pat test_2ops_float_imm/${insn} ${insn}
		generate 100 test_src_float_dir_src_dst_int_reg.pat test_2ops_float_dir/${insn} ${insn}
	done
	
	for insn in cmpf; do
		generate 100 test_2ops_src_int_reg_src_int_reg.pat test_2ops_float_reg/int_int/${insn} ${insn}
		generate 100 test_2ops_src_float_reg_src_int_reg.pat test_2ops_float_reg/float_int/${insn} ${insn}
		generate 100 test_2ops_src_int_reg_src_float_reg.pat test_2ops_float_reg/int_float/${insn} ${insn}
		generate 100 test_2ops_src_float_reg_src_float_reg.pat test_2ops_float_reg/float_float/${insn} ${insn}
		generate_test_src_float_indir_src_dst_int_reg 100 test_2ops_float_indir ${insn}
		generate_test_src_float_imm_dst_float_reg 100 test_2ops_src_float_imm_src_dst_int_reg.pat test_2ops_float_imm/${insn} ${insn}
		generate 100 test_src_float_dir_src_dst_int_reg.pat test_2ops_float_dir/${insn} ${insn}
	done
	
	for insn in fix; do
		generate 100 test_2ops_src_int_reg_dst_int_reg.pat test_2ops_float_reg/${insn} ${insn}
	done

	for insn in float; do
		generate 100 test_2ops_src_float_reg_dst_int_reg.pat test_2ops_float_reg/${insn} ${insn}
		generate_test_src_int_indir_dst_int_reg 100 test_2ops_float_indir ${insn}
		generate_test_src_int_imm_dst_int_reg 1 test_2ops_float_imm ${insn}
		generate 100 test_src_int_dir_src_dst_int_reg.pat test_2ops_float_dir/${insn} ${insn}
	done
	fi

	if [ "${float_2ops_instructions}" = "yes" ]; then
	for insn in absf negf norm rnd; do
		generate_src_float_indir_dst_float_reg 100 2ops_float_indir ${insn}
		generate 10000 2ops_src_float_reg_dst_float_reg.pat 2ops_float_reg/${insn} ${insn}
		generate_src_float_imm_dst_float_reg 1 2ops_src_float_imm_dst_float_reg.pat 2ops_float_imm/${insn} ${insn}
		generate 100 src_float_dir_dst_float_reg.pat 2ops_float_dir/${insn} ${insn}
	done

	for insn in addf subf subrf mpyf; do
		generate_src_float_indir_src_dst_float_reg 100 2ops_float_indir ${insn}
		generate 10000 2ops_src_float_reg_src_dst_float_reg.pat 2ops_float_reg/${insn} ${insn}
		generate_src_float_imm_dst_float_reg 100 2ops_src_float_imm_src_dst_float_reg.pat 2ops_float_imm/${insn} ${insn}
		generate 100 src_float_dir_src_dst_float_reg.pat 2ops_float_dir/${insn} ${insn}
	done

	for insn in cmpf; do
		generate_src_float_indir_src_float_reg 100 2ops_float_indir ${insn}
		generate 10000 2ops_src_float_reg_src_float_reg.pat 2ops_float_reg/${insn} ${insn}
		generate_src_float_imm_dst_float_reg 100 2ops_src_float_imm_src_float_reg.pat 2ops_float_imm/${insn} ${insn}
		generate 100 src_float_dir_src_float_reg.pat 2ops_float_dir/${insn} ${insn}
	done

	for insn in fix; do
		# NOTE: the fix instruction sets the result into an integer register, but we test it also
		# against a float register to check the how the extra bits are conserved/modified
		generate_src_float_indir_dst_int_reg 100 2ops_float_indir ${insn}
		generate_src_float_indir_src_dst_float_reg_ext 100 2ops_float_indir ${insn}
		generate 10000 2ops_src_float_reg_dst_int_reg.pat 2ops_float_reg/${insn} ${insn}
		generate 10000 2ops_src_float_reg_src_dst_float_reg.pat 2ops_float_reg/${insn}_dst_ext ${insn}
		generate_src_float_imm_dst_float_reg 1 2ops_src_float_imm_dst_int_reg.pat 2ops_float_imm/${insn} ${insn}
		generate_src_float_imm_dst_float_reg 100 2ops_src_float_imm_src_dst_float_reg.pat 2ops_float_imm/${insn}_dst_ext ${insn}
		generate 100 src_float_dir_src_dst_float_reg.pat 2ops_float_dir/${insn} ${insn}
		generate 100 2ops_src_float_buf_dst_int_reg_ar_update_ordering.pat 2ops_float_indir/${insn}_ar_update_ordering ${insn}
	done

	for insn in float; do
		generate_src_int_indir_dst_float_reg 100 2ops_float_indir ${insn}
		generate 10000 2ops_src_int_reg_dst_float_reg.pat 2ops_float_reg/${insn} ${insn}
		generate_src_int_imm_dst_float_reg 1 2ops_float_imm ${insn}
		generate 100 src_int_dir_dst_float_reg.pat 2ops_float_dir/${insn} ${insn}
		# generate 100 2ops_src_int_reg_dst_float_reg_ar_update_ordering.pat 2ops_float_indir/${insn}_ar_update_ordering ${insn}
	done
	fi

# END: float 2 operands instructions                     #
##########################################################

##########################################################
# START: float 3 operands instructions                   #

	if [ "${float_3ops_instructions}" = "yes" ]; then
	for insn in addf3 subf3 mpyf3; do 
		# reg reg
		generate 100 3ops_src_float_reg_src_float_reg_dst_float_reg.pat 3ops_float_reg_reg/${insn} ${insn}

		# reg buf
		generate 100 3ops_src_float_reg_src_float_buf_dst_float_reg.pat 3ops_float_reg_buf/${insn} ${insn}

		# buf reg
		generate 100 3ops_src_float_buf_src_float_reg_dst_float_reg.pat 3ops_float_buf_reg/${insn} ${insn}

		# buf buf
		generate 100 3ops_src_float_buf_src_float_buf_dst_float_reg.pat 3ops_float_buf_buf/${insn} ${insn}
		generate 100 3ops_src_float_buf_src_float_buf_dst_float_reg_ar_update_ordering.pat 3ops_float_buf_buf/${insn}_ar_update_ordering ${insn}
	done

	for insn in cmpf3; do
		# reg reg
		generate 100 3ops_src_float_reg_src_float_reg.pat 3ops_float_reg_reg/${insn} ${insn}

		# reg buf
		generate 100 3ops_src_float_reg_src_float_buf.pat 3ops_float_reg_buf/${insn} ${insn}

		# buf reg
		generate 100 3ops_src_float_buf_src_float_reg.pat 3ops_float_buf_reg/${insn} ${insn}

		# buf buf
		generate 100 3ops_src_float_buf_src_float_buf.pat 3ops_float_buf_buf/${insn} ${insn}
		generate 100 3ops_src_float_buf_src_float_buf_ar_update_ordering.pat 3ops_float_buf_buf/${insn}_ar_update_ordering ${insn}
	done
	fi

# END: float 3 operands instructions                     #
##########################################################

##########################################################
# START: float parallel instructions                     #

	if [ "${float_parallel_instruction}" = "yes" ]; then

		# ldf || ldf
		# buf_buf
		generate 100 par_src_float_buf_dst_float_reg_src_float_buf_dst_float_reg.pat parallel_float/ldf_ldf/buf_buf ldf ldf
		generate 100 par_src_float_buf_dst_float_reg_src_float_buf_dst_float_reg_ar_update_ordering.pat parallel_float/ldf_ldf_ar_update_ordering/buf_buf ldf ldf
		# reg_buf
		generate 100 par_src_float_reg_dst_float_reg_src_float_buf_dst_float_reg.pat parallel_float/ldf_ldf/reg_buf ldf ldf

		# ldf || stf
		# buf
		generate 100 par_src_float_buf_dst_float_reg_src_float_reg_dst_float_buf.pat parallel_float/ldf_stf/buf_buf ldf stf
		generate 100 par_src_float_buf_dst_float_reg_src_float_reg_dst_float_buf_ar_update_ordering.pat parallel_float/ldf_stf_ar_update_ordering/buf_buf ldf stf
		# reg
		generate 100 par_src_float_reg_dst_float_reg_src_float_reg_dst_float_buf.pat parallel_float/ldf_stf/reg_buf ldf stf

		# stf || stf
		# buf_buf
		generate 100 par_src_float_reg_dst_float_buf_src_float_reg_dst_float_buf.pat parallel_float/stf_stf/buf_buf stf stf
		generate 100 par_src_float_reg_dst_float_buf_src_float_reg_dst_float_buf_ar_update_ordering.pat parallel_float/stf_stf_ar_update_ordering/buf_buf stf stf
		# reg_buf
		generate 100 par_src_float_reg_dst_float_reg_src_float_reg_dst_float_buf.pat parallel_float/stf_stf/reg_buf stf stf

		for insn in fix; do
			for insn2 in sti; do
				# buf reg
				generate 100 par_src_float_buf_src_dst_float_reg_src_float_reg_dst_int_buf.pat parallel_float/${insn}_${insn2}/buf_reg ${insn} ${insn2}
				generate 100 par_src_float_buf_dst_int_reg_src_float_reg_dst_int_buf_ar_update_ordering.pat parallel_float/${insn}_${insn2}_ar_update_ordering/buf_reg ${insn} ${insn2}

				# reg reg
				generate 100 par_src_float_reg_src_dst_float_reg_src_float_reg_dst_int_buf.pat parallel_float/${insn}_${insn2}/reg_reg ${insn} ${insn2}
			done
		done

		for insn in float; do
			for insn2 in stf; do
				# buf reg
				generate 100 par_src_int_buf_dst_float_reg_src_float_reg_dst_float_buf.pat parallel_float/${insn}_${insn2}/buf_reg ${insn} ${insn2}
				generate 100 par_src_int_buf_dst_float_reg_src_float_reg_dst_float_buf_ar_update_ordering.pat parallel_float/${insn}_${insn2}_ar_update_ordering/buf_reg ${insn} ${insn2}

				# reg reg
				generate 100 par_src_int_reg_dst_float_reg_src_float_reg_dst_float_buf.pat parallel_float/${insn}_${insn2}/reg_reg ${insn} ${insn2}
			done
		done

		for insn in absf negf; do
			for insn2 in stf; do
				# buf reg
				generate 100 par_src_float_buf_dst_float_reg_src_float_reg_dst_float_buf.pat parallel_float/${insn}_${insn2}/buf_reg ${insn} ${insn2}
				generate 100 par_src_float_buf_dst_float_reg_src_float_reg_dst_float_buf_ar_update_ordering.pat parallel_float/${insn}_${insn2}_ar_update_ordering/buf_reg ${insn} ${insn2}

				# reg reg
				generate 100 par_src_float_reg_dst_float_reg_src_flaot_reg_dst_float_buf.pat parallel_float/${insn}_${insn2}/reg_reg ${insn} ${insn2}
			done
		done

		for insn in addf3 subf3 mpyf3; do
			for insn2 in stf; do
				# reg buf
				generate 100 par_src_float_reg_src_float_buf_dst_float_reg_src_float_reg_dst_float_buf.pat parallel_float/${insn}_${insn2}/reg_buf ${insn} ${insn2}
				generate 100 par_src_float_reg_src_float_buf_dst_float_reg_src_float_reg_dst_float_buf_ar_update_ordering.pat parallel_float/${insn}_${insn2}_ar_update_ordering/reg_buf ${insn} ${insn2}

				# reg reg
				generate 100 par_src_float_reg_src_float_reg_dst_float_reg_src_float_reg_dst_float_buf.pat parallel_float/${insn}_${insn2}/reg_reg ${insn} ${insn2}
			done
		done

		for insn in mpyf3; do
			for insn2 in addf3 subf3; do
				# buf_buf_reg_reg
				generate 100 par_src_float_buf_src_float_buf_dst_float_reg_src_float_reg_src_float_reg_dst_float_reg.pat parallel_float/${insn}_${insn2}/buf_buf_reg_reg ${insn} ${insn2}
				generate 100 par_src_float_buf_src_float_buf_dst_float_reg_src_float_reg_src_float_reg_dst_float_reg_ar_update_ordering.pat parallel_float/${insn}_${insn2}_ar_update_ordering/buf_buf_reg_reg ${insn} ${insn2}
				# buf_reg_reg_reg
				generate 100 par_src_float_buf_src_float_reg_dst_float_reg_src_float_reg_src_float_reg_dst_float_reg.pat parallel_float/${insn}_${insn2}/buf_reg_reg_reg ${insn} ${insn2}
				# reg_buf_reg_reg
				generate 100 par_src_float_reg_src_float_buf_dst_float_reg_src_float_reg_src_float_reg_dst_float_reg.pat parallel_float/${insn}_${insn2}/reg_buf_reg_reg ${insn} ${insn2}
				# reg_reg_reg_reg
				generate 100 par_src_float_reg_src_float_reg_dst_float_reg_src_float_reg_src_float_reg_dst_float_reg.pat parallel_float/${insn}_${insn2}/reg_reg_reg_reg ${insn} ${insn2}
				# buf_reg_buf_reg
				generate 100 par_src_float_buf_src_float_reg_dst_float_reg_src_float_buf_src_float_reg_dst_float_reg.pat parallel_float/${insn}_${insn2}/buf_reg_buf_reg ${insn} ${insn2}
				generate 100 par_src_float_buf_src_float_reg_dst_float_reg_src_float_buf_src_float_reg_dst_float_reg_ar_update_ordering.pat parallel_float/${insn}_${insn2}_ar_update_ordering/buf_reg_buf_reg ${insn} ${insn2}
				# reg_reg_buf_reg
				generate 100 par_src_float_reg_src_float_reg_dst_float_reg_src_float_buf_src_float_reg_dst_float_reg.pat parallel_float/${insn}_${insn2}/reg_reg_buf_reg ${insn} ${insn2}
				# reg_reg_buf_buf
				generate 100 par_src_float_reg_src_float_reg_dst_float_reg_src_float_buf_src_float_buf_dst_float_reg.pat parallel_float/${insn}_${insn2}/reg_reg_buf_buf ${insn} ${insn2} 
				generate 100 par_src_float_reg_src_float_reg_dst_float_reg_src_float_buf_src_float_buf_dst_float_reg_ar_update_ordering.pat parallel_float/${insn}_${insn2}_ar_update_ordering/reg_reg_buf_buf ${insn} ${insn2} 
				# reg_reg_reg_buf
				generate 100 par_src_float_reg_src_float_reg_dst_float_reg_src_float_reg_src_float_buf_dst_float_reg.pat parallel_float/${insn}_${insn2}/reg_reg_reg_buf ${insn} ${insn2} 
				# buf_reg_reg_buf
				generate 100 par_src_float_buf_src_float_reg_dst_float_reg_src_float_reg_src_float_buf_dst_float_reg.pat parallel_float/${insn}_${insn2}/buf_reg_reg_buf ${insn} ${insn2} 
				generate 100 par_src_float_buf_src_float_reg_dst_float_reg_src_float_reg_src_float_buf_dst_float_reg_ar_update_ordering.pat parallel_float/${insn}_${insn2}_ar_update_ordering/buf_reg_reg_buf ${insn} ${insn2} 
			done
		done
	fi

# END: float parallel instructions                       #
##########################################################
}

function generate_test_src_float_imm_dst_float_reg {
	local random_count=$1
	local pattern=$2
	local subdir=$3
	local insn=$4
	local -a float_values=("0.0" "1.0" "-1.0" "1.5" "-1.5" "2.5594e2" "7.8125e-3" "-7.8163e-3" "-2.56e2")

	for float in ${float_values[@]}; do
		generate ${random_count} ${pattern} "${subdir}/${float}" ${insn} "${float}"
	done
}

function generate_src_float_imm_dst_float_reg {
	local random_count=$1
	local pattern=$2
	local subdir=$3
	local insn=$4
	local -a float_values=("0.0" "1.0" "-1.0" "1.5" "-1.5" "2.5594e2" "7.8125e-3" "-7.8163e-3" "-2.56e2")

	for float in ${float_values[@]}; do
		generate ${random_count} ${pattern} "${subdir}/${float}" ${insn} "${float}"
	done
}

function generate_test_src_int_imm_dst_int_reg 
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	local -a imm_values=("0" "1" "-1" "-32768" "32767")

	for imm in ${imm_values[@]}; do
		generate ${random_count} test_2ops_src_int_imm_dst_int_reg.pat "${subdir}/${insn}/${imm}" ${insn} "${imm}"
	done
}

function generate_src_int_imm_dst_float_reg 
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	local -a imm_values=("0" "1" "-1" "-32768" "32767")

	for imm in ${imm_values[@]}; do
		generate ${random_count} 2ops_src_int_imm_dst_float_reg.pat "${subdir}/${insn}/${imm}" ${insn} "${imm}"
	done
}

function generate_src_int_imm_dst_int_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	local -a imm_values=("0" "1" "-1" "-32768" "32767")

	for imm in ${imm_values[@]}; do
		generate ${random_count} 2ops_src_int_imm_dst_int_reg.pat "${subdir}/${insn}/${imm}" ${insn} "${imm}"
	done
}


function generate_src_int_imm_src_int_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	local -a imm_values=("0" "1" "-1" "-32768" "32767")

	for imm in ${imm_values[@]}; do
		generate ${random_count} 2ops_src_int_imm_src_int_reg.pat "${subdir}/${insn}/${imm}" ${insn} "${imm}"
	done
}

function generate_src_int_imm_src_dst_int_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	local -a imm_values=("0" "1" "-1" "-32768" "32767")

	for imm in ${imm_values[@]}; do
		generate ${random_count} 2ops_src_int_imm_src_dst_int_reg.pat "${subdir}/${insn}/${imm}" ${insn} "${imm}"
	done
}

function generate_src_int_indir_dst_int_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} src_int_indir_predisp_add_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_add ${insn} 1
	generate ${random_count} src_int_indir_predisp_sub_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_sub ${insn} 1
	generate ${random_count} src_int_indir_predisp_add_mod_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_add_mod ${insn} 1
	generate ${random_count} src_int_indir_predisp_sub_mod_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postdisp_add_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_add_mod ${insn} 1
	generate ${random_count} src_int_indir_postdisp_sub_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postdisp_add_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_indir_postdisp_sub_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_indir_postdisp_add_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_indir_postdisp_sub_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn

	generate ${random_count} src_int_indir_preind_ir0_add_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_add ${insn}
	generate ${random_count} src_int_indir_preind_ir0_sub_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub ${insn}
	generate ${random_count} src_int_indir_preind_ir0_add_mod_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_add_mod ${insn} 1
	generate ${random_count} src_int_indir_preind_ir0_sub_mod_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir0_add_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir0_sub_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_mod ${insn} 1 
	generate ${random_count} src_int_indir_postind_ir0_add_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir0_sub_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir0_add_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir0_sub_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_int_indir_preind_ir1_add_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_add ${insn}
	generate ${random_count} src_int_indir_preind_ir1_sub_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub ${insn}
	generate ${random_count} src_int_indir_preind_ir1_add_mod_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_add_mod ${insn} 1
	generate ${random_count} src_int_indir_preind_ir1_sub_mod_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir1_add_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir1_sub_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_mod ${insn} 1 
	generate ${random_count} src_int_indir_postind_ir1_add_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir1_sub_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir1_add_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir1_sub_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_int_indir_dst_int_reg.pat ${subdir}/${insn}/indir ${insn}
	generate ${random_count} src_int_indir_postind_ir0_add_bit_rev_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_bit_rev_mod ${insn}
}

function generate_test_src_int_indir_dst_int_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} test_src_int_indir_predisp_add_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_add ${insn} 1
}

function generate_src_int_indir_dst_float_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} src_int_indir_predisp_add_dst_float_reg.pat ${subdir}/${insn}/indir_predisp_add ${insn} 1
	generate ${random_count} src_int_indir_predisp_sub_dst_float_reg.pat ${subdir}/${insn}/indir_predisp_sub ${insn} 1
	generate ${random_count} src_int_indir_predisp_add_mod_dst_float_reg.pat ${subdir}/${insn}/indir_predisp_add_mod ${insn} 1
	generate ${random_count} src_int_indir_predisp_sub_mod_dst_float_reg.pat ${subdir}/${insn}/indir_predisp_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postdisp_add_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_add_mod ${insn} 1
	generate ${random_count} src_int_indir_postdisp_sub_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postdisp_add_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_indir_postdisp_sub_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_indir_postdisp_add_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_indir_postdisp_sub_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn

	generate ${random_count} src_int_indir_preind_ir0_add_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_add ${insn}
	generate ${random_count} src_int_indir_preind_ir0_sub_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub ${insn}
	generate ${random_count} src_int_indir_preind_ir0_add_mod_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_add_mod ${insn} 1
	generate ${random_count} src_int_indir_preind_ir0_sub_mod_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir0_add_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir0_sub_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_mod ${insn} 1 
	generate ${random_count} src_int_indir_postind_ir0_add_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir0_sub_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir0_add_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir0_sub_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_int_indir_preind_ir1_add_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_add ${insn}
	generate ${random_count} src_int_indir_preind_ir1_sub_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub ${insn}
	generate ${random_count} src_int_indir_preind_ir1_add_mod_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_add_mod ${insn} 1
	generate ${random_count} src_int_indir_preind_ir1_sub_mod_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir1_add_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir1_sub_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_mod ${insn} 1 
	generate ${random_count} src_int_indir_postind_ir1_add_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir1_sub_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir1_add_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir1_sub_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_int_indir_dst_float_reg.pat ${subdir}/${insn}/indir ${insn}
	generate ${random_count} src_int_indir_postind_ir0_add_bit_rev_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_bit_rev_mod ${insn}
}

function generate_src_int_reg_dst_int_indir
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} src_int_reg_dst_int_indir_predisp_add.pat ${subdir}/${insn}/indir_predisp_add ${insn} 1
	generate ${random_count} src_int_reg_dst_int_indir_predisp_sub.pat ${subdir}/${insn}/indir_predisp_sub ${insn} 1
	generate ${random_count} src_int_reg_dst_int_indir_predisp_add_mod.pat ${subdir}/${insn}/indir_predisp_add_mod ${insn} 1
	generate ${random_count} src_int_reg_dst_int_indir_predisp_sub_mod.pat ${subdir}/${insn}/indir_predisp_sub_mod ${insn} 1
	generate ${random_count} src_int_reg_dst_int_indir_postdisp_add_mod.pat ${subdir}/${insn}/indir_postdisp_add_mod ${insn} 1
	generate ${random_count} src_int_reg_dst_int_indir_postdisp_sub_mod.pat ${subdir}/${insn}/indir_postdisp_sub_mod ${insn} 1
	generate ${random_count} src_int_reg_dst_int_indir_postdisp_add_circ_mod.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_reg_dst_int_indir_postdisp_sub_circ_mod.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_reg_dst_int_indir_postdisp_add_circ_mod.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_reg_dst_int_indir_postdisp_sub_circ_mod.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn

	generate ${random_count} src_int_reg_dst_int_indir_preind_ir0_add.pat ${subdir}/${insn}/indir_preind_ir0_add ${insn}
	generate ${random_count} src_int_reg_dst_int_indir_preind_ir0_sub.pat ${subdir}/${insn}/indir_preind_ir0_sub ${insn}
	generate ${random_count} src_int_reg_dst_int_indir_preind_ir0_add_mod.pat ${subdir}/${insn}/indir_preind_ir0_add_mod ${insn} 1
	generate ${random_count} src_int_reg_dst_int_indir_preind_ir0_sub_mod.pat ${subdir}/${insn}/indir_preind_ir0_sub_mod ${insn} 1
	generate ${random_count} src_int_reg_dst_int_indir_postind_ir0_add_mod.pat ${subdir}/${insn}/indir_postind_ir0_add_mod ${insn} 1
	generate ${random_count} src_int_reg_dst_int_indir_postind_ir0_sub_mod.pat ${subdir}/${insn}/indir_postind_ir0_sub_mod ${insn} 1 
	generate ${random_count} src_int_reg_dst_int_indir_postind_ir0_add_circ_mod.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_reg_dst_int_indir_postind_ir0_sub_circ_mod.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_reg_dst_int_indir_postind_ir0_add_circ_mod.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_reg_dst_int_indir_postind_ir0_sub_circ_mod.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_int_reg_dst_int_indir_preind_ir1_add.pat ${subdir}/${insn}/indir_preind_ir1_add ${insn}
	generate ${random_count} src_int_reg_dst_int_indir_preind_ir1_sub.pat ${subdir}/${insn}/indir_preind_ir1_sub ${insn}
	generate ${random_count} src_int_reg_dst_int_indir_preind_ir1_add_mod.pat ${subdir}/${insn}/indir_preind_ir1_add_mod ${insn} 1
	generate ${random_count} src_int_reg_dst_int_indir_preind_ir1_sub_mod.pat ${subdir}/${insn}/indir_preind_ir1_sub_mod ${insn} 1
	generate ${random_count} src_int_reg_dst_int_indir_postind_ir1_add_mod.pat ${subdir}/${insn}/indir_postind_ir1_add_mod ${insn} 1
	generate ${random_count} src_int_reg_dst_int_indir_postind_ir1_sub_mod.pat ${subdir}/${insn}/indir_postind_ir1_sub_mod ${insn} 1 
	generate ${random_count} src_int_reg_dst_int_indir_postind_ir1_add_circ_mod.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_reg_dst_int_indir_postind_ir1_sub_circ_mod.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_reg_dst_int_indir_postind_ir1_add_circ_mod.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_reg_dst_int_indir_postind_ir1_sub_circ_mod.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_int_reg_dst_int_indir.pat ${subdir}/${insn}/indir ${insn}
	generate ${random_count} src_int_reg_dst_int_indir_postind_ir0_add_bit_rev_mod.pat ${subdir}/${insn}/indir_postind_ir0_add_bit_rev_mod ${insn}
}


function generate_src_int_indir_src_int_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} src_int_indir_predisp_add_src_int_reg.pat ${subdir}/${insn}/indir_predisp_add ${insn} 1
	generate ${random_count} src_int_indir_predisp_sub_src_int_reg.pat ${subdir}/${insn}/indir_predisp_sub ${insn} 1
	generate ${random_count} src_int_indir_predisp_add_mod_src_int_reg.pat ${subdir}/${insn}/indir_predisp_add_mod ${insn} 1
	generate ${random_count} src_int_indir_predisp_sub_mod_src_int_reg.pat ${subdir}/${insn}/indir_predisp_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postdisp_add_mod_src_int_reg.pat ${subdir}/${insn}/indir_postdisp_add_mod ${insn} 1
	generate ${random_count} src_int_indir_postdisp_sub_mod_src_int_reg.pat ${subdir}/${insn}/indir_postdisp_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postdisp_add_circ_mod_src_int_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_indir_postdisp_sub_circ_mod_src_int_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_indir_postdisp_add_circ_mod_src_int_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_indir_postdisp_sub_circ_mod_src_int_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn

	generate ${random_count} src_int_indir_preind_ir0_add_src_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_add ${insn}
	generate ${random_count} src_int_indir_preind_ir0_sub_src_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub ${insn}
	generate ${random_count} src_int_indir_preind_ir0_add_mod_src_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_add_mod ${insn} 1
	generate ${random_count} src_int_indir_preind_ir0_sub_mod_src_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir0_add_mod_src_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir0_sub_mod_src_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_mod ${insn} 1 
	generate ${random_count} src_int_indir_postind_ir0_add_circ_mod_src_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir0_sub_circ_mod_src_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir0_add_circ_mod_src_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir0_sub_circ_mod_src_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_int_indir_preind_ir1_add_src_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_add ${insn}
	generate ${random_count} src_int_indir_preind_ir1_sub_src_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub ${insn}
	generate ${random_count} src_int_indir_preind_ir1_add_mod_src_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_add_mod ${insn} 1
	generate ${random_count} src_int_indir_preind_ir1_sub_mod_src_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir1_add_mod_src_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir1_sub_mod_src_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_mod ${insn} 1 
	generate ${random_count} src_int_indir_postind_ir1_add_circ_mod_src_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir1_sub_circ_mod_src_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir1_add_circ_mod_src_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir1_sub_circ_mod_src_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_int_indir_src_int_reg.pat ${subdir}/${insn}/indir ${insn}
	generate ${random_count} src_int_indir_postind_ir0_add_bit_rev_mod_src_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_bit_rev_mod ${insn}
}

function generate_src_int_indir_src_dst_int_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} src_int_indir_predisp_add_src_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_add ${insn} 1
	generate ${random_count} src_int_indir_predisp_sub_src_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_sub ${insn} 1
	generate ${random_count} src_int_indir_predisp_add_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_add_mod ${insn} 1
	generate ${random_count} src_int_indir_predisp_sub_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postdisp_add_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_add_mod ${insn} 1
	generate ${random_count} src_int_indir_postdisp_sub_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postdisp_add_circ_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_indir_postdisp_sub_circ_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_indir_postdisp_add_circ_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_int_indir_postdisp_sub_circ_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn

	generate ${random_count} src_int_indir_preind_ir0_add_src_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_add ${insn}
	generate ${random_count} src_int_indir_preind_ir0_sub_src_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub ${insn}
	generate ${random_count} src_int_indir_preind_ir0_add_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_add_mod ${insn} 1
	generate ${random_count} src_int_indir_preind_ir0_sub_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir0_add_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir0_sub_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_mod ${insn} 1 
	generate ${random_count} src_int_indir_postind_ir0_add_circ_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir0_sub_circ_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir0_add_circ_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir0_sub_circ_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_int_indir_preind_ir1_add_src_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_add ${insn}
	generate ${random_count} src_int_indir_preind_ir1_sub_src_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub ${insn}
	generate ${random_count} src_int_indir_preind_ir1_add_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_add_mod ${insn} 1
	generate ${random_count} src_int_indir_preind_ir1_sub_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir1_add_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_mod ${insn} 1
	generate ${random_count} src_int_indir_postind_ir1_sub_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_mod ${insn} 1 
	generate ${random_count} src_int_indir_postind_ir1_add_circ_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir1_sub_circ_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir1_add_circ_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_int_indir_postind_ir1_sub_circ_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_int_indir_src_dst_int_reg.pat ${subdir}/${insn}/indir ${insn}
	generate ${random_count} src_int_indir_postind_ir0_add_bit_rev_mod_src_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_bit_rev_mod ${insn}
}

function generate_src_float_indir_dst_float_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} src_float_indir_predisp_add_dst_float_reg.pat ${subdir}/${insn}/indir_predisp_add ${insn} 1
	generate ${random_count} src_float_indir_predisp_sub_dst_float_reg.pat ${subdir}/${insn}/indir_predisp_sub ${insn} 1
	generate ${random_count} src_float_indir_predisp_add_mod_dst_float_reg.pat ${subdir}/${insn}/indir_predisp_add_mod ${insn} 1
	generate ${random_count} src_float_indir_predisp_sub_mod_dst_float_reg.pat ${subdir}/${insn}/indir_predisp_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_add_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_sub_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_add_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_sub_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_add_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_sub_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn

	generate ${random_count} src_float_indir_preind_ir0_add_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_add ${insn}
	generate ${random_count} src_float_indir_preind_ir0_sub_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub ${insn}
	generate ${random_count} src_float_indir_preind_ir0_add_mod_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_add_mod ${insn} 1
	generate ${random_count} src_float_indir_preind_ir0_sub_mod_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir0_add_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir0_sub_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_mod ${insn} 1 
	generate ${random_count} src_float_indir_postind_ir0_add_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_sub_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_add_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_sub_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_float_indir_preind_ir1_add_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_add ${insn}
	generate ${random_count} src_float_indir_preind_ir1_sub_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub ${insn}
	generate ${random_count} src_float_indir_preind_ir1_add_mod_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_add_mod ${insn} 1
	generate ${random_count} src_float_indir_preind_ir1_sub_mod_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir1_add_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir1_sub_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_mod ${insn} 1 
	generate ${random_count} src_float_indir_postind_ir1_add_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_sub_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_add_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_sub_circ_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_float_indir_dst_float_reg.pat ${subdir}/${insn}/indir ${insn}
	generate ${random_count} src_float_indir_postind_ir0_add_bit_rev_mod_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_bit_rev_mod ${insn}
}

function generate_src_float_indir_dst_int_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} src_float_indir_predisp_add_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_add ${insn} 1
	generate ${random_count} src_float_indir_predisp_sub_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_sub ${insn} 1
	generate ${random_count} src_float_indir_predisp_add_mod_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_add_mod ${insn} 1
	generate ${random_count} src_float_indir_predisp_sub_mod_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_add_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_sub_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_add_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_sub_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_add_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_sub_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn

	generate ${random_count} src_float_indir_preind_ir0_add_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_add ${insn}
	generate ${random_count} src_float_indir_preind_ir0_sub_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub ${insn}
	generate ${random_count} src_float_indir_preind_ir0_add_mod_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_add_mod ${insn} 1
	generate ${random_count} src_float_indir_preind_ir0_sub_mod_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir0_add_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir0_sub_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_mod ${insn} 1 
	generate ${random_count} src_float_indir_postind_ir0_add_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_sub_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_add_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_sub_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_float_indir_preind_ir1_add_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_add ${insn}
	generate ${random_count} src_float_indir_preind_ir1_sub_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub ${insn}
	generate ${random_count} src_float_indir_preind_ir1_add_mod_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_add_mod ${insn} 1
	generate ${random_count} src_float_indir_preind_ir1_sub_mod_dst_int_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir1_add_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir1_sub_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_mod ${insn} 1 
	generate ${random_count} src_float_indir_postind_ir1_add_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_sub_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_add_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_sub_circ_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_float_indir_dst_int_reg.pat ${subdir}/${insn}/indir ${insn}
	generate ${random_count} src_float_indir_postind_ir0_add_bit_rev_mod_dst_int_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_bit_rev_mod ${insn}
}

function generate_src_float_reg_dst_float_indir
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} src_float_reg_dst_float_indir_predisp_add.pat ${subdir}/${insn}/indir_predisp_add ${insn} 1
	generate ${random_count} src_float_reg_dst_float_indir_predisp_sub.pat ${subdir}/${insn}/indir_predisp_sub ${insn} 1
	generate ${random_count} src_float_reg_dst_float_indir_predisp_add_mod.pat ${subdir}/${insn}/indir_predisp_add_mod ${insn} 1
	generate ${random_count} src_float_reg_dst_float_indir_predisp_sub_mod.pat ${subdir}/${insn}/indir_predisp_sub_mod ${insn} 1
	generate ${random_count} src_float_reg_dst_float_indir_postdisp_add_mod.pat ${subdir}/${insn}/indir_postdisp_add_mod ${insn} 1
	generate ${random_count} src_float_reg_dst_float_indir_postdisp_sub_mod.pat ${subdir}/${insn}/indir_postdisp_sub_mod ${insn} 1
	generate ${random_count} src_float_reg_dst_float_indir_postdisp_add_circ_mod.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_reg_dst_float_indir_postdisp_sub_circ_mod.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_reg_dst_float_indir_postdisp_add_circ_mod.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_reg_dst_float_indir_postdisp_sub_circ_mod.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn

	generate ${random_count} src_float_reg_dst_float_indir_preind_ir0_add.pat ${subdir}/${insn}/indir_preind_ir0_add ${insn}
	generate ${random_count} src_float_reg_dst_float_indir_preind_ir0_sub.pat ${subdir}/${insn}/indir_preind_ir0_sub ${insn}
	generate ${random_count} src_float_reg_dst_float_indir_preind_ir0_add_mod.pat ${subdir}/${insn}/indir_preind_ir0_add_mod ${insn} 1
	generate ${random_count} src_float_reg_dst_float_indir_preind_ir0_sub_mod.pat ${subdir}/${insn}/indir_preind_ir0_sub_mod ${insn} 1
	generate ${random_count} src_float_reg_dst_float_indir_postind_ir0_add_mod.pat ${subdir}/${insn}/indir_postind_ir0_add_mod ${insn} 1
	generate ${random_count} src_float_reg_dst_float_indir_postind_ir0_sub_mod.pat ${subdir}/${insn}/indir_postind_ir0_sub_mod ${insn} 1 
	generate ${random_count} src_float_reg_dst_float_indir_postind_ir0_add_circ_mod.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_reg_dst_float_indir_postind_ir0_sub_circ_mod.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_reg_dst_float_indir_postind_ir0_add_circ_mod.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_reg_dst_float_indir_postind_ir0_sub_circ_mod.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_float_reg_dst_float_indir_preind_ir1_add.pat ${subdir}/${insn}/indir_preind_ir1_add ${insn}
	generate ${random_count} src_float_reg_dst_float_indir_preind_ir1_sub.pat ${subdir}/${insn}/indir_preind_ir1_sub ${insn}
	generate ${random_count} src_float_reg_dst_float_indir_preind_ir1_add_mod.pat ${subdir}/${insn}/indir_preind_ir1_add_mod ${insn} 1
	generate ${random_count} src_float_reg_dst_float_indir_preind_ir1_sub_mod.pat ${subdir}/${insn}/indir_preind_ir1_sub_mod ${insn} 1
	generate ${random_count} src_float_reg_dst_float_indir_postind_ir1_add_mod.pat ${subdir}/${insn}/indir_postind_ir1_add_mod ${insn} 1
	generate ${random_count} src_float_reg_dst_float_indir_postind_ir1_sub_mod.pat ${subdir}/${insn}/indir_postind_ir1_sub_mod ${insn} 1 
	generate ${random_count} src_float_reg_dst_float_indir_postind_ir1_add_circ_mod.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_reg_dst_float_indir_postind_ir1_sub_circ_mod.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_reg_dst_float_indir_postind_ir1_add_circ_mod.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_reg_dst_float_indir_postind_ir1_sub_circ_mod.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_float_reg_dst_float_indir.pat ${subdir}/${insn}/indir ${insn}
	generate ${random_count} src_float_reg_dst_float_indir_postind_ir0_add_bit_rev_mod.pat ${subdir}/${insn}/indir_postind_ir0_add_bit_rev_mod ${insn}
}


function generate_test_src_float_indir_src_dst_float_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} test_src_float_indir_predisp_add_src_dst_float_reg.pat ${subdir}/${insn}/indir_predisp_add ${insn} 1
}

function generate_test_src_float_indir_src_dst_int_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} test_src_float_indir_predisp_add_src_dst_int_reg.pat ${subdir}/${insn}/indir_predisp_add ${insn} 1
}

function generate_src_float_indir_src_dst_float_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} src_float_indir_predisp_add_src_dst_float_reg.pat ${subdir}/${insn}/indir_predisp_add ${insn} 1
	generate ${random_count} src_float_indir_predisp_sub_src_dst_float_reg.pat ${subdir}/${insn}/indir_predisp_sub ${insn} 1
	generate ${random_count} src_float_indir_predisp_add_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_predisp_add_mod ${insn} 1
	generate ${random_count} src_float_indir_predisp_sub_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_predisp_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_add_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_sub_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_add_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_sub_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_add_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_sub_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn

	generate ${random_count} src_float_indir_preind_ir0_add_src_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_add ${insn}
	generate ${random_count} src_float_indir_preind_ir0_sub_src_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub ${insn}
	generate ${random_count} src_float_indir_preind_ir0_add_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_add_mod ${insn} 1
	generate ${random_count} src_float_indir_preind_ir0_sub_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir0_add_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir0_sub_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_mod ${insn} 1 
	generate ${random_count} src_float_indir_postind_ir0_add_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_sub_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_add_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_sub_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_float_indir_preind_ir1_add_src_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_add ${insn}
	generate ${random_count} src_float_indir_preind_ir1_sub_src_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub ${insn}
	generate ${random_count} src_float_indir_preind_ir1_add_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_add_mod ${insn} 1
	generate ${random_count} src_float_indir_preind_ir1_sub_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir1_add_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir1_sub_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_mod ${insn} 1 
	generate ${random_count} src_float_indir_postind_ir1_add_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_sub_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_add_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_sub_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_float_indir_src_dst_float_reg.pat ${subdir}/${insn}/indir ${insn}
	generate ${random_count} src_float_indir_postind_ir0_add_bit_rev_mod_src_dst_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_bit_rev_mod ${insn}
}

function generate_src_float_indir_src_dst_float_reg_ext
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} src_float_indir_predisp_add_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_predisp_add ${insn} 1
	generate ${random_count} src_float_indir_predisp_sub_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_predisp_sub ${insn} 1
	generate ${random_count} src_float_indir_predisp_add_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_predisp_add_mod ${insn} 1
	generate ${random_count} src_float_indir_predisp_sub_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_predisp_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_add_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postdisp_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_sub_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postdisp_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_add_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postdisp_add_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_sub_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postdisp_sub_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_add_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postdisp_add_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_sub_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postdisp_sub_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn

	generate ${random_count} src_float_indir_preind_ir0_add_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_preind_ir0_add ${insn}
	generate ${random_count} src_float_indir_preind_ir0_sub_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_preind_ir0_sub ${insn}
	generate ${random_count} src_float_indir_preind_ir0_add_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_preind_ir0_add_mod ${insn} 1
	generate ${random_count} src_float_indir_preind_ir0_sub_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_preind_ir0_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir0_add_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postind_ir0_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir0_sub_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postind_ir0_sub_mod ${insn} 1 
	generate ${random_count} src_float_indir_postind_ir0_add_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postind_ir0_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_sub_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postind_ir0_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_add_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postind_ir0_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_sub_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postind_ir0_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_float_indir_preind_ir1_add_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_preind_ir1_add ${insn}
	generate ${random_count} src_float_indir_preind_ir1_sub_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_preind_ir1_sub ${insn}
	generate ${random_count} src_float_indir_preind_ir1_add_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_preind_ir1_add_mod ${insn} 1
	generate ${random_count} src_float_indir_preind_ir1_sub_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_preind_ir1_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir1_add_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postind_ir1_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir1_sub_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postind_ir1_sub_mod ${insn} 1 
	generate ${random_count} src_float_indir_postind_ir1_add_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postind_ir1_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_sub_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postind_ir1_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_add_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postind_ir1_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_sub_circ_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postind_ir1_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_float_indir_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir ${insn}
	generate ${random_count} src_float_indir_postind_ir0_add_bit_rev_mod_src_dst_float_reg.pat ${subdir}/${insn}_dst_ext/indir_postind_ir0_add_bit_rev_mod ${insn}
}

function generate_src_float_indir_src_float_reg
{
	local random_count=$1
	local subdir=$2
	local insn=$3
	generate ${random_count} src_float_indir_predisp_add_src_float_reg.pat ${subdir}/${insn}/indir_predisp_add ${insn} 1
	generate ${random_count} src_float_indir_predisp_sub_src_float_reg.pat ${subdir}/${insn}/indir_predisp_sub ${insn} 1
	generate ${random_count} src_float_indir_predisp_add_mod_src_float_reg.pat ${subdir}/${insn}/indir_predisp_add_mod ${insn} 1
	generate ${random_count} src_float_indir_predisp_sub_mod_src_float_reg.pat ${subdir}/${insn}/indir_predisp_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_add_mod_src_float_reg.pat ${subdir}/${insn}/indir_postdisp_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_sub_mod_src_float_reg.pat ${subdir}/${insn}/indir_postdisp_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postdisp_add_circ_mod_src_float_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_sub_circ_mod_src_float_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_4 4 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_add_circ_mod_src_float_reg.pat ${subdir}/${insn}/indir_postdisp_add_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn
	generate ${random_count} src_float_indir_postdisp_sub_circ_mod_src_float_reg.pat ${subdir}/${insn}/indir_postdisp_sub_circ_mod_bk_5 5 7 ${insn} 1 # block size, block mask, insn

	generate ${random_count} src_float_indir_preind_ir0_add_src_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_add ${insn}
	generate ${random_count} src_float_indir_preind_ir0_sub_src_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub ${insn}
	generate ${random_count} src_float_indir_preind_ir0_add_mod_src_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_add_mod ${insn} 1
	generate ${random_count} src_float_indir_preind_ir0_sub_mod_src_float_reg.pat ${subdir}/${insn}/indir_preind_ir0_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir0_add_mod_src_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir0_sub_mod_src_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_mod ${insn} 1 
	generate ${random_count} src_float_indir_postind_ir0_add_circ_mod_src_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_sub_circ_mod_src_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_add_circ_mod_src_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir0_sub_circ_mod_src_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_float_indir_preind_ir1_add_src_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_add ${insn}
	generate ${random_count} src_float_indir_preind_ir1_sub_src_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub ${insn}
	generate ${random_count} src_float_indir_preind_ir1_add_mod_src_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_add_mod ${insn} 1
	generate ${random_count} src_float_indir_preind_ir1_sub_mod_src_float_reg.pat ${subdir}/${insn}/indir_preind_ir1_sub_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir1_add_mod_src_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_mod ${insn} 1
	generate ${random_count} src_float_indir_postind_ir1_sub_mod_src_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_mod ${insn} 1 
	generate ${random_count} src_float_indir_postind_ir1_add_circ_mod_src_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_sub_circ_mod_src_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_4 4 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_add_circ_mod_src_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_add_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn
	generate ${random_count} src_float_indir_postind_ir1_sub_circ_mod_src_float_reg.pat ${subdir}/${insn}/indir_postind_ir1_sub_circ_mod_bk_5 5 7 ${insn} # block size, block mask, insn

	generate ${random_count} src_float_indir_src_float_reg.pat ${subdir}/${insn}/indir ${insn}
	generate ${random_count} src_float_indir_postind_ir0_add_bit_rev_mod_src_float_reg.pat ${subdir}/${insn}/indir_postind_ir0_add_bit_rev_mod ${insn}
}

################################################################################################

PATTERN_SUBDIR=patterns
INPUT_SUBDIR=inputs
DEPS_MK=/tmp/factorial_${RANDOM}
COMPILE_MK=/tmp/factorial_${RANDOM}
RND_MK=/tmp/factorial_${RANDOM}
EXECUTE_MK=/tmp/factorial_${RANDOM}
SIMULATE_MK=/tmp/factorial_${RANDOM}
DIFF_MK=/tmp/factorial_${RANDOM}
DOC_DEPS_MK=/tmp/factorial_${RANDOM}
DOC_MK=/tmp/factorial_${RANDOM}
CHECK_DEPS_MK=/tmp/factorial_${RANDOM}
CHECK_MK=/tmp/factorial_${RANDOM}
DIST_MK=/tmp/factorial_${RANDOM}
DIST_DEPS_MK=/tmp/factorial_${RANDOM}
CLEAN_BIN_MK=/tmp/factorial_${RANDOM}
CLEAN_TEST_OBJ_MK=/tmp/factorial_${RANDOM}
CLEAN_MAIN_OBJ_MK=/tmp/factorial_${RANDOM}
CLEAN_GEL_MK=/tmp/factorial_${RANDOM}
CLEAN_DIFF_MK=/tmp/factorial_${RANDOM}
CLEAN_CHECK_MK=/tmp/factorial_${RANDOM}
CLEAN_REF_MK=/tmp/factorial_${RANDOM}
CLEAN_SIM_MK=/tmp/factorial_${RANDOM}
CLEAN_RND_MK=/tmp/factorial_${RANDOM}
CLEAN_DOC_MK=/tmp/factorial_${RANDOM}
CLEAN_TEST_ASM_MK=/tmp/factorial_${RANDOM}
CLEAN_MAIN_C_MK=/tmp/factorial_${RANDOM}

while (( "$#" )); do
	if [ $1 = "--force" ] || [ $1 = "-f" ]; then
		FORCE=-f
		shift
		continue
	fi

	if [ $1 = "--help"] || [ $1 = "-h" ]; then
		echo "Usage: "`basename $0`" [--force]"
		exit 0
	fi
done

printf "Generating factorial plan into auxiliary Makefile (Makefile.aux). Please wait...\n"

if [[ `uname` =~ "MINGW" ]]; then
	EXESUFFIX=.exe
else
	EXESUFFIX=
fi

function generate
{
	local RANDOM_COUNT=$1
	local PATTERN_SHORT_FILENAME=$2
	local PATTERN_FILENAME=${PATTERN_SUBDIR}/${PATTERN_SHORT_FILENAME}
	local OUTPUT_SUBDIRNAME=$3
	local NUM_FIXED_ARGS=3

	local INPUT_FILENAME=${INPUT_SUBDIR}/`echo "${PATTERN_SHORT_FILENAME}" | sed 's/\.pat$//g'`.txt

	printf "${OUTPUT_SUBDIRNAME}/test.asm: ${PATTERN_FILENAME}\n" >> ${DEPS_MK}
	printf "\tmkdir -p ${OUTPUT_SUBDIRNAME}\n" >> ${DEPS_MK}
	printf "\t./generator\$(EXESUFFIX) -u -f -p ${PATTERN_FILENAME} -o ${OUTPUT_SUBDIRNAME} -- " >> ${DEPS_MK}
	local i=0
	for arg in "$@"
	do
			if test ${i} -ge ${NUM_FIXED_ARGS}; then
				printf " \"${arg}\"" >> ${DEPS_MK}
			fi
			i=$((${i}+1))
	done
	printf "\n\n" >> ${DEPS_MK}

	printf "${OUTPUT_SUBDIRNAME}/test.tex: ${PATTERN_FILENAME} ${INPUT_FILENAME}\n" >> ${DEPS_MK}
	printf "\tmkdir -p ${OUTPUT_SUBDIRNAME}\n" >> ${DEPS_MK}
	printf "\t./generator\$(EXESUFFIX) -d -c ${RANDOM_COUNT} -f -p ${PATTERN_FILENAME} -i ${INPUT_FILENAME} -o ${OUTPUT_SUBDIRNAME} -- " >> ${DEPS_MK}
	i=0
	for arg in "$@"
	do
			if test ${i} -ge ${NUM_FIXED_ARGS}; then
				printf " \"${arg}\"" >> ${DEPS_MK}
			fi
			i=$((${i}+1))
	done
	printf "\n\n" >> ${DEPS_MK}

	printf "${OUTPUT_SUBDIRNAME}/main.c: ${PATTERN_FILENAME}\n" >> ${DEPS_MK}
	printf "\tmkdir -p ${OUTPUT_SUBDIRNAME}\n" >> ${DEPS_MK}
	printf "\t./generator\$(EXESUFFIX) -t -f -p ${PATTERN_FILENAME} -i ${INPUT_FILENAME} -o ${OUTPUT_SUBDIRNAME}\n\n" >> ${DEPS_MK}

	printf "${OUTPUT_SUBDIRNAME}/random.txt: ${PATTERN_FILENAME}\n" >> ${DEPS_MK}
	printf "\tmkdir -p ${OUTPUT_SUBDIRNAME}\n" >> ${DEPS_MK}
	printf "\t./generator\$(EXESUFFIX) -r -c ${RANDOM_COUNT} -f -p ${PATTERN_FILENAME} -o ${OUTPUT_SUBDIRNAME}\n\n" >> ${DEPS_MK}

	printf "${OUTPUT_SUBDIRNAME}/main.obj : ${OUTPUT_SUBDIRNAME}/main.c\n" >> ${DEPS_MK}
	printf "\tcd ${OUTPUT_SUBDIRNAME}; \$(CC) \$(CFLAGS) \$(INCLUDE_DIRECTIVE) main.c\n\n" >> ${DEPS_MK}

	printf "${OUTPUT_SUBDIRNAME}/test.obj : ${OUTPUT_SUBDIRNAME}/test.asm\n" >> ${DEPS_MK}
	printf "\tcd ${OUTPUT_SUBDIRNAME}; \$(AS) test.asm \$(ASFLAGS)\n\n" >> ${DEPS_MK}

	printf "${OUTPUT_SUBDIRNAME}/test.out: firm.obj ${OUTPUT_SUBDIRNAME}/test.obj ${OUTPUT_SUBDIRNAME}/main.obj\n" >> ${DEPS_MK}
	printf "\t\$(LD) \$(LDFLAGS) -o\$@ \$(LDSCRIPT) \$^ \$(LINK_DIRECTIVE) \$(LIBS)\n\n" >> ${DEPS_MK}

	printf "${OUTPUT_SUBDIRNAME}/output.sim: ${OUTPUT_SUBDIRNAME}/test.out ${INPUT_FILENAME} ${OUTPUT_SUBDIRNAME}/random.txt c31boot.out sim_config.xml \$(SIMULATE)\n" >> ${DEPS_MK}
	printf "\t@if [ -z \"\$(SIMULATE)\" ]; then echo \"Please set path to the TMS320C3X simulator executable into variable SIMULATE.\"; exit 255; fi\n" >> ${DEPS_MK}
	printf "\t\$(SIMULATE) -c sim_config.xml ${OUTPUT_SUBDIRNAME}/test.out\n" >> ${DEPS_MK}
	printf "\tmv -f ${OUTPUT_SUBDIRNAME}/output.txt ${OUTPUT_SUBDIRNAME}/output.sim\n\n" >> ${DEPS_MK}

	printf "${OUTPUT_SUBDIRNAME}/output.ref: ${OUTPUT_SUBDIRNAME}/run.gel ${OUTPUT_SUBDIRNAME}/test.out ${INPUT_FILENAME} ${OUTPUT_SUBDIRNAME}/random.txt\n" >> ${DEPS_MK}
	printf "\tif [ -z \"\$(EXECUTE)\" ]; then echo \"Please set path to the Code Composer executable into variable EXECUTE.\"; exit 255; fi\n" >> ${DEPS_MK}
	printf "\t\$(EXECUTE) ${OUTPUT_SUBDIRNAME}/run.gel\n" >> ${DEPS_MK}
	printf "\tsleep \$(DELAY_BETWEEN_EXECUTE)\n" >> ${DEPS_MK}
	printf "\tif [ \`uname\` == \"Darwin\" ]; then gsed \'s/\\\\x0D\$\$//\' ${OUTPUT_SUBDIRNAME}/output.txt > \$@; else sed 's/\\\\x0D\$\$//' ${OUTPUT_SUBDIRNAME}/output.txt > \$@; fi\n" >> ${DEPS_MK}
	printf "\t\$(RM) ${OUTPUT_SUBDIRNAME}/output.txt\n" >> ${DEPS_MK}

	printf "${OUTPUT_SUBDIRNAME}/output.diff: ${OUTPUT_SUBDIRNAME}/output.ref ${OUTPUT_SUBDIRNAME}/output.sim\n" >> ${DEPS_MK}
	printf "\tdiff \$^ > \$@ || true\n\n" >> ${DEPS_MK}

	printf "${OUTPUT_SUBDIRNAME}/output.check: ${OUTPUT_SUBDIRNAME}/output.diff\n" >> ${DEPS_MK}
	printf "\t@printf \"${OUTPUT_SUBDIRNAME} \" > \$@; if [ -f \$< ] && [ ! -s \$< ]; then printf \"PASSED\" >> \$@; else printf \"FAILED\" >> \$@; fi; printf \"\\\\n\" >> \$@\n\n" >> ${DEPS_MK}

	printf "${OUTPUT_SUBDIRNAME}/run.gel:\n" >> ${DEPS_MK}
	printf "\t@printf \"StartUp()\\\\r\\\\n\" > \$@\n" >> ${DEPS_MK}
	printf "\t@printf \"{\\\\r\\\\n\" >> \$@\n" >> ${DEPS_MK}
	printf "\t@printf \"\\\\tGEL_Reset();\\\\r\\\\n\" >> \$@\n" >> ${DEPS_MK}
	printf "\t@printf \"\\\\tGEL_Load(\\\\\"${OUTPUT_SUBDIRNAME}/test.out\\\\\");\\\\r\\\\n\" >> \$@\n" >> ${DEPS_MK}
	printf "\t@printf \"\\\\tGEL_Go(C\\\$\$\\\$\$EXIT);\\\\r\\\\n\" >> \$@\n" >> ${DEPS_MK}
	printf "\t@printf \"\\\\tGEL_Exit();\\\\r\\\\n\" >> \$@\n" >> ${DEPS_MK}
	printf "\t@printf \"}\\\\r\\\\n\" >> \$@\n\n" >> ${DEPS_MK}

	printf " ${OUTPUT_SUBDIRNAME}/test.out" >> ${COMPILE_MK}

	printf " ${OUTPUT_SUBDIRNAME}/random.txt" >> ${RND_MK}

	printf " ${OUTPUT_SUBDIRNAME}/output.ref" >> ${EXECUTE_MK}

	printf "\tcp -f ${PATTERN_FILENAME} \$(DIST_DIR)/${PATTERN_FILENAME}\n\tcp -f ${INPUT_FILENAME} \$(DIST_DIR)/${INPUT_FILENAME}\n\tmkdir -p \$(DIST_DIR)/${OUTPUT_SUBDIRNAME}\n\tcp -f ${OUTPUT_SUBDIRNAME}/output.ref \$(DIST_DIR)/${OUTPUT_SUBDIRNAME}/output.ref\n" >> ${DIST_MK}

	printf " ${OUTPUT_SUBDIRNAME}/output.sim" >> ${SIMULATE_MK}

	printf " ${OUTPUT_SUBDIRNAME}/output.diff" >> ${DIFF_MK}

	printf " ${OUTPUT_SUBDIRNAME}/output.check" >> ${CHECK_DEPS_MK}

	printf " ${OUTPUT_SUBDIRNAME}/test.tex" >> ${DOC_DEPS_MK}

	printf " ${OUTPUT_SUBDIRNAME}/output.ref ${INPUT_FILENAME}" >> ${DIST_DEPS_MK}

	printf "\t@cat ${OUTPUT_SUBDIRNAME}/test.tex >> \$@\n" >> ${DOC_MK}

	printf "\t@cat ${OUTPUT_SUBDIRNAME}/output.check\n" >> ${CHECK_MK}

	printf "\t\$(RM) ${OUTPUT_SUBDIRNAME}/run.gel\n" >> ${CLEAN_GEL_MK}
	printf "\t\$(RM) ${OUTPUT_SUBDIRNAME}/output.check\n" >> ${CLEAN_CHECK_MK}
	printf "\t\$(RM) ${OUTPUT_SUBDIRNAME}/output.diff\n" >> ${CLEAN_DIFF_MK}
	printf "\t\$(RM) ${OUTPUT_SUBDIRNAME}/output.ref\n" >> ${CLEAN_REF_MK}
	printf "\t\$(RM) ${OUTPUT_SUBDIRNAME}/output.sim\n" >> ${CLEAN_SIM_MK}
	printf "\t\$(RM) ${OUTPUT_SUBDIRNAME}/test.out\n" >> ${CLEAN_BIN_MK}
	printf "\t\$(RM) ${OUTPUT_SUBDIRNAME}/test.obj\n" >> ${CLEAN_TEST_OBJ_MK}
	printf "\t\$(RM) ${OUTPUT_SUBDIRNAME}/main.obj\n" >> ${CLEAN_MAIN_OBJ_MK}
	printf "\t\$(RM) ${OUTPUT_SUBDIRNAME}/test.asm\n" >> ${CLEAN_TEST_ASM_MK}
	printf "\t\$(RM) ${OUTPUT_SUBDIRNAME}/main.c\n" >> ${CLEAN_MAIN_C_MK}
	printf "\t\$(RM) ${OUTPUT_SUBDIRNAME}/random.txt\n" >> ${CLEAN_RND_MK}
	printf "\t\$(RM) ${OUTPUT_SUBDIRNAME}/test.tex\n" >> ${CLEAN_DOC_MK}

	unit_tests_count=$((${unit_tests_count}+1))
	instruction_count=$((${instruction_count}+${RANDOM_COUNT}))
	printf "\033[K\033[50D${unit_tests_count} programs (${instruction_count} unit tests)"
}

printf "compile: " > ${COMPILE_MK}
printf "rnd: " > ${RND_MK}
printf "execute: " > ${EXECUTE_MK}
printf "simulate: " > ${SIMULATE_MK}
printf "diff: " > ${DIFF_MK}
printf "check: " > ${CHECK_DEPS_MK}
printf "dist: Makefile factorial.sh generator.cc unit_tests.lds sim_config.xml firm.asm unit_tests.pdf" > ${DIST_DEPS_MK}

printf "\t@echo \"Generating \$@. Please wait...\"\n" > ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\documentclass[a4paper,10pt]{article}\\\\n\" > \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\voffset -2.0cm\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\hoffset -0.75cm\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\textheight 26cm\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\textwidth 17cm\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\topmargin 0.0cm\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\oddsidemargin 0.0cm\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\evensidemargin 0.0cm\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\usepackage{hyperref}\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\hypersetup{\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"  pdfauthor   = {Gilles Mouchard},\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"  pdftitle    = {UNISIM\\ TMS320C3X Unit tests comprehensive list},\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"  pdfsubject  = {UNISIM\\ TMS320C3X Unit tests comprehensive list},\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"  pdfkeywords = {regression test,validation,unit test},\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"  pdfcreator  = {rubber},\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"  pdfproducer = {rubber}\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"}\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\title{UNISIM\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\ TMS320C3X Unit tests comprehensive list}\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\author{Gilles Mouchard \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\Daniel Gracia P\\\\\\\\\\\\\\\\'erez \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\Reda Nouacer \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\CEA List}\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\date{\`date \"+%%Y, %%b %%e\"\`}\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\begin{document}\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\maketitle\\\\n\" >> \$@\n" >> ${DOC_MK}

printf "" > ${CHECK_MK}
printf "\t@if [ -z \"\$(DIST_DIR)\" ]; then echo \"Please set path to destination directory into variable DIST_DIR.\"; exit 255; fi\n" > ${DIST_MK}
printf "\tmkdir -p \$(DIST_DIR)\n" >> ${DIST_MK}
printf "\tmkdir -p \$(DIST_DIR)/inputs\n" >> ${DIST_MK}
printf "\tmkdir -p \$(DIST_DIR)/patterns\n" >> ${DIST_MK}
printf "\tcp -f README \$(DIST_DIR)/README\n" >> ${DIST_MK}
printf "\tcp -f COPYING \$(DIST_DIR)/COPYING\n" >> ${DIST_MK}
printf "\tcp -f Makefile \$(DIST_DIR)/Makefile\n" >> ${DIST_MK}
printf "\tcp -f factorial.sh \$(DIST_DIR)/factorial.sh\n" >> ${DIST_MK}
printf "\tcp -f generator.cc \$(DIST_DIR)/generator.cc\n" >> ${DIST_MK}
printf "\tcp -f unit_tests.lds \$(DIST_DIR)/unit_tests.lds\n" >> ${DIST_MK}
printf "\tcp -f sim_config.xml \$(DIST_DIR)/sim_config.xml\n" >> ${DIST_MK}
printf "\tcp -f firm.asm \$(DIST_DIR)/firm.asm\n" >> ${DIST_MK}
printf "\tcp -f c31boot.asm \$(DIST_DIR)/c31boot.asm\n" >> ${DIST_MK}
printf "\tcp -f c31boot.lds \$(DIST_DIR)/c31boot.lds\n" >> ${DIST_MK}
printf "\tcp -f unit_tests.pdf \$(DIST_DIR)/unit_tests.pdf\n" >> ${DIST_MK}
printf "unit_tests.tex: " >> ${DOC_DEPS_MK}
printf "cleanbin:\n" > ${CLEAN_BIN_MK}
printf "cleantestobj:\n" > ${CLEAN_TEST_OBJ_MK}
printf "cleanmainobj:\n" > ${CLEAN_MAIN_OBJ_MK}
printf "cleangel:\n" > ${CLEAN_GEL_MK}
printf "cleandiff:\n" > ${CLEAN_DIFF_MK}
printf "cleancheck:\n" > ${CLEAN_CHECK_MK}
printf "cleanref:\n" > ${CLEAN_REF_MK}
printf "cleansim:\n" > ${CLEAN_SIM_MK}
printf "cleantestasm:\n" > ${CLEAN_TEST_ASM_MK}
printf "cleanmainc:\n" > ${CLEAN_MAIN_C_MK}
printf "cleanrnd:\n" > ${CLEAN_RND_MK}
printf "cleandoc:\n\t\$(RM) unit_tests.tex\n" > ${CLEAN_DOC_MK}

factorial

printf "\n" >> ${COMPILE_MK}
printf "\n" >> ${RND_MK}
printf "\n" >> ${EXECUTE_MK}
printf "\n" >> ${SIMULATE_MK}
printf "\n" >> ${DIFF_MK}
printf "\t@printf \"\\\\\\\\\\\\\\\\end{document}\\\\n\" >> \$@\n" >> ${DOC_MK}
printf "\n" >> ${DOC_DEPS_MK}
printf "\n" >> ${CHECK_MK}
printf "\n" >> ${CHECK_DEPS_MK}
printf "\n" >> ${DIST_MK}
printf "\n" >> ${DIST_DEPS_MK}
printf "\n" >> ${CLEAN_BIN_MK}
printf "\n" >> ${CLEAN_TEST_OBJ_MK}
printf "\n" >> ${CLEAN_MAIN_OBJ_MK}
printf "\n" >> ${CLEAN_GEL_MK}
printf "\n" >> ${CLEAN_DIFF_MK}
printf "\n" >> ${CLEAN_CHECK_MK}
printf "\n" >> ${CLEAN_REF_MK}
printf "\n" >> ${CLEAN_SIM_MK}
printf "\n" >> ${CLEAN_TEST_ASM_MK}
printf "\n" >> ${CLEAN_MAIN_C_MK}
printf "\n" >> ${CLEAN_RND_MK}
printf "\n" >> ${CLEAN_DOC_MK}

printf "cleanobj: cleantestobj cleanmainobj\n" > Makefile.aux
printf "\t\$(RM) firm.obj\n" >> Makefile.aux
printf "cleansrc: cleantestasm cleanmainc\n" >> Makefile.aux
printf "doc: unit_tests.pdf\n" >> Makefile.aux
printf "unit_tests.pdf: unit_tests.tex\n" >> Makefile.aux
printf "\trubber -d \$<\n" >> Makefile.aux
printf "status:\n\t@echo \"${unit_tests_count} programs (${instruction_count} unit tests)\"\n" >> Makefile.aux

cat ${DEPS_MK} ${COMPILE_MK} ${RND_MK} ${EXECUTE_MK} ${SIMULATE_MK} ${DIFF_MK} ${CHECK_DEPS_MK} ${CHECK_MK} ${DOC_DEPS_MK} ${DOC_MK} ${DIST_DEPS_MK} ${DIST_MK} ${CLEAN_BIN_MK} ${CLEAN_TEST_OBJ_MK} ${CLEAN_MAIN_OBJ_MK} ${CLEAN_GEL_MK} ${CLEAN_DIFF_MK} ${CLEAN_CHECK_MK} ${CLEAN_REF_MK} ${CLEAN_SIM_MK} ${CLEAN_TEST_ASM_MK} ${CLEAN_MAIN_C_MK} ${CLEAN_RND_MK} ${CLEAN_DOC_MK} >> Makefile.aux

# temporary files
rm -f ${COMPILE_MK}
rm -f ${RND_MK}
rm -f ${EXECUTE_MK}
rm -f ${SIMULATE_MK}
rm -f ${DIFF_MK}
rm -f ${DOC_DEPS_MK}
rm -f ${DOC_MK}
rm -f ${CHECK_DEPS_MK}
rm -f ${CHECK_MK}
rm -f ${DIST_DEPS_MK}
rm -f ${DIST_MK}
rm -f ${DEPS_MK}
rm -f ${CLEAN_BIN_MK}
rm -f ${CLEAN_TEST_OBJ_MK}
rm -f ${CLEAN_MAIN_OBJ_MK}
rm -f ${CLEAN_GEL_MK}
rm -f ${CLEAN_DIFF_MK}
rm -f ${CLEAN_CHECK_MK}
rm -f ${CLEAN_REF_MK}
rm -f ${CLEAN_SIM_MK}
rm -f ${CLEAN_TEST_ASM_MK}
rm -f ${CLEAN_MAIN_C_MK}
rm -f ${CLEAN_RND_MK}
rm -f ${CLEAN_DOC_MK}

printf "\n"
